'''                                             MAIN PROGRAM VERONICA ROBOT

Main classes definition for the Vernonica Robot HMI along with functions required for the MHI to tun properly.
Contains:   - HMI_Veronica class, main application class. Handles frame management and application shutdown. Also retrieves display mode
            - Start_Page class, handles the starting page structure and the directions towards other pages
            -Main_Page class, handles the Command Page structure, as well as the Movement Speed, Position and Angle commands and the display
             mode selection.
            -Display_Page class, handles the display page structure along with the data update and data display. Displayed as a secondary
             page when opening the Command Page and is sent to the background when any other window is open.
            -Help_Page class, handles the help page structure, displaying how to use the HMI.
            -Analysis_Page class, handles the analysis page structure, displaying a live graph of data as well as mean values and a save functionnality.

            -Retrieve_Data function, handles in real time, as a second thread, the verification of incoming data. If new data is received, calls the update method from Display_Page class.

NOTES:  -Make sure to check the shell working directory using os.getcwd(). If the repertory isn't the where the code is stored, make sure to replace it using os.chdir(r"paths") where "paths" corresponds to where the code is stored.

        -Simple tests can be run using the Run_file.py code

        -DO NOT RUN ON PYZO (can display errors when trying to load images). The program has been tested using Windows Powershell:

                                                        cd "paths"
                                                        py .\Test.py


Written on Python v3.8.2
Author : FRESLIER Clément, 26/04/2020
'''


'''Libraries
------------------------------------------------------------------------------'''
import os
from functools import partial
from PIL import ImageTk, Image
from datetime import datetime, timedelta
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
import matplotlib.animation as animation
from time import sleep
from matplotlib import style


try:
    # Python 2
    import Tkinter as tk
    from Tkinter.messagebox import *
    import Tkinter.ttk as ttk

except ImportError:
    # Python 3
    import tkinter as tk
    from tkinter.messagebox import *
    import tkinter.ttk as ttk


''' Constants
------------------------------------------------------------------------------'''
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
style.use("ggplot")

#figure in which the graph will be displayed
fig=Figure(figsize=(7,6), dpi=100,constrained_layout=True)

#subplots for the figure with 2 y axis
ax1=fig.add_subplot(111)
ax2=ax1.twinx()




''' Application Class
------------------------------------------------------------------------------'''
class HMI_Veronica(tk.Tk):



    def __init__ (self,*args,**kwargs):

        '''Master class attributes
------------------------------------------------------------------------------'''

        #Initializing the application window
        tk.Tk.__init__(self, *args, **kwargs)
        self.title( 'Robot Veronica Interface Utilisateur')

        #Doesn't allow the user to resize the window (blocks the use of the "fullscreen" button)
        self.resizable(False,False)


        #Frame acting as a container for all useful frames
        container = tk.Frame(self)
        container.pack(side="top", fill="both",expand=True)
        container.grid_rowconfigure(0, weight=1)
        container.grid_columnconfigure(0, weight=1)


        #Variable definition
        self.State=True                             #System state (False=standby)
        self.M_Speed=0                              #Movement speed command
        self.Angle=tk.StringVar(self)               #Angular position command
        self.Position=tk.StringVar(self)            #Position command
        self.Angle.set('0')
        self.Position.set('0')
        self.data=[]                                #data list
        self.x_index=0                              #index of current date value when plotting data
        self.avg_Temp=0                             #mean temperature
        self.avg_Hyg=0                              #mean hygrometry level

        #Managing frames
        self.frames = {}


        for F in (Start_Page, Main_Page, Help_Page,Analysis_Page):

            frame = F(container, self)
            self.frames[F] = frame
            frame.grid(row=0, column=0, sticky="nsew")

        #raises the start page at initialization
        self.show_frame(Start_Page)


        self.protocol("WM_DELETE_WINDOW", self.callback)


    '''Class Methods
------------------------------------------------------------------------------'''
    def show_frame(self, cont):

        '''Raises the selected frame to top level, defines for each frame the window size as well as its position on the screen
           Creates the Display Page if none already exists. Deletes it when it is closed so that it can be re-opened later on.
           Inputs: self, cont -> cont is the selected frame
           Outputs: None
           Author: FRESLIER Clément 22/04/2020
        '''

        frame = self.frames[cont]
        frame.tkraise()

        #Managing Start Page
        if (self.frames[cont].__class__==Start_Page):

            #screen size
            w,h=self.winfo_screenwidth(),self.winfo_screenheight()
            x,y=int(w/2-230),int(h/2-155)

            #Positionning the page at the center of the screen with given size
            self.geometry('460x310+'+str(x)+'+'+str(y))

            #Sending Display Page to the background if it exists
            if self.frames[Main_Page].Data_Page:
                self.frames[Main_Page].Data_Page.withdraw()

        #Managing Help Page
        if (self.frames[cont].__class__==Help_Page):

            #screen size
            w,h=self.winfo_screenwidth(),self.winfo_screenheight()
            x,y=int(w/2-605),int(h/2-255)

            #Positionning the page at the center of the screen with given size
            self.geometry('1210x510+'+str(x)+'+'+str(y))

            #Sending Display Page to the background if it exists
            if self.frames[Main_Page].Data_Page:
                self.frames[Main_Page].Data_Page.withdraw()

        #Managing Command Page
        if (self.frames[cont].__class__==Main_Page):

            #Window geometry
            h=self.winfo_screenheight()
            y=int(h/2-302)

            self.geometry('845x532+650+'+str(y))

            #if no Display Page already exists
            if (self.frames[cont].Data_Page==None):

                #Creating Display Frame
                self.frames[cont].Data_Page=Display_Page(self,self)
                Data_Page=self.frames[cont].Data_Page
                Data_Page.geometry('500x532+100+'+str(y))
                Data_Page.title('Données')

            #if a Display Page already exists but is sent to the background
            elif self.frames[cont].Data_Page:


                self.frames[cont].Data_Page.deiconify()

                Data_Page=self.frames[cont].Data_Page


            def close_Data_Frame():

                '''When closing the Data Page using the "close window" button, sets back the Data_Page variable type as None, so
                that it can be re-opened when re-opening the command window
                Inputs: None
                Outputs: None
                Author: FRESLIER Clément 26/04/2020
                '''

                self.frames[cont].Data_Page.destroy()
                self.frames[cont].Data_Page=None

            Data_Page.protocol("WM_DELETE_WINDOW", close_Data_Frame)




        if (self.frames[cont].__class__==Analysis_Page):

            w,h=self.winfo_screenwidth(),self.winfo_screenheight()
            x,y=int(w/2-600),int(h/2-350)

            self.geometry('1200x700+'+str(x)+'+'+str(y))

            #Sending Display Page to the background if it exists
            if self.frames[Main_Page].Data_Page:
                self.frames[Main_Page].Data_Page.withdraw()



    def Shutdown(self):

        '''Shuts the robot down by setting the State boolean as False and quits the HMI application (asks user before doing so)
           Inputs: self
           Outputs: None
           Author : FRESLIER Clément, 25/03/2020                              '''

        Invisible_root=tk.Tk().withdraw()
        if askyesno("Warning","Fermer la fenêtre de commande entraînera l'arrêt du robot, voulez-vous continuer ?"):
            self.destroy()
            self.quit()
            self.State=False



    def callback(self):

        '''Asks before shutting down the program using the default "close window" icon
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 22/04/2020                               '''

        if askokcancel("Fermer l'application", "Voulez-vous vraiment fermer l'application?"):
            self.destroy()
            self.quit()
            State=False



    def Get_mode(self):

        '''Returns the display mode selected (data as a function of time or position)
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 26/04/2020                               '''

        return self.frames[Main_Page].var.get()




'''Subclass containing the starting page of the application
------------------------------------------------------------------------------'''
class Start_Page(tk.Frame):



    def __init__(self, master, controller):

        '''Class Attributes
------------------------------------------------------------------------------'''

        #Initializing the frame
        tk.Frame.__init__(self,master,width=460,height=310)
        self.master=master


        #Start Page Background Image
        self.bg_img=tk.Canvas(self,width=460,height=310)
        self.img=ImageTk.PhotoImage(file=r"images\Robot_Veronica_Img.png")
        self.background=self.bg_img.create_image(230,155,image=self.img,anchor="center")
        self.bg_img.place(x=0,y=0)


        #Main Page Title
        self.label = ttk.Label(self, text="Robot Veronica Interface Utilisateur",font=('Helvetica',21),anchor='center')
        self.label.place(x=230,y=0,anchor="n",relwidth=1)


        #Main Page Menu : buttons allowing the user to navigate between help page and command page and quit application button
        interface_button = ttk.Button(self, text="Interface Utilisateur",
                            command=lambda: controller.show_frame(Main_Page))
        interface_button.place(x=230,y=310,anchor="s")


        help_button = ttk.Button(self, text="Aide",
                            command=lambda: controller.show_frame(Help_Page))
        help_button.place(x=0,y=310,anchor='sw')


        exit_button=ttk.Button(self,text='Arrêt Robot',command=controller.Shutdown)
        exit_button.place(x=460,y=310,anchor='se')



'''Subclass containing the command frame and the display frame
------------------------------------------------------------------------------'''
class Main_Page(tk.Frame):



    def __init__(self, master, controller):

        '''Class Attributes
------------------------------------------------------------------------------'''

        tk.Frame.__init__(self, master)


        #controller=master class(HMI_Veronica here), used to call methods inherited from that class, master=tk.Frame
        self.controller=controller
        self.master=master


        #Display Page, None value means there isn't any display page created yet
        self.Data_Page=None


        #Command Page Title
        Command_Label=tk.Label(self,text="Panneau de commande")
        Command_Label.grid(row=0,column=1,pady=20)
        Command_Label.config(font=("Courrier",30))


        #Back to Home Page button
        Home_Button= tk.Button(self, text="Retour au Menu Principal",
                            command=lambda: controller.show_frame(Start_Page))
        Home_Button.grid(row=15,column=0,sticky="SW")


        #Opens Help Page
        Help_Button = tk.Button(self, text="Aide",
                            command=lambda: controller.show_frame(Help_Page))
        Help_Button.grid(row=15,column=2,sticky='SE')


        Open_Analysis = tk.Button(self, text="Analyse des données",
                            command=lambda: controller.show_frame(Analysis_Page))
        Open_Analysis.grid(row=15,column=1,sticky="S")


        #Exits Application
        exit_button=tk.Button(self,text='Arrêt Robot',command=controller.Shutdown)
        exit_button.grid(row=15,column=3)



        #Setting Command Speed and validating entry. The set value is stored in M_Speed attribute (inherited from master class)
        MS_Label=tk.Label(self,text="Commande de vitesse \n (Une commande entre 0 et 9 \n imposera une vitesse de 10m/s) ")
        MS_Label.grid(row=2,column=0,pady=20)
        self.MS_Slider=tk.Scale(self,from_=0,to=30,tickinterval=10,resolution=1,orient='horizontal') #slider, units in cm/s
        self.MS_Slider.grid(row=2,column=1,pady=20)
        MS_Set=tk.Button(self,text="Valider",command=self.Set_MSpeed)
        MS_Set.grid(row=2, column=2, pady=20,padx=20)



        #Setting Position Command and validating entry. The value is stored using Position.get() method
        Position_Label=tk.Label(self,text="Commande de position (en cm)")
        Position_Label.grid(row=4,column=0,pady=20)
        Position_Entry=tk.Entry(self,textvariable=self.controller.Position)
        Position_Entry.grid(row=4,column=1,padx=5,pady=20)
        POS_Set=tk.Button(self,text="Valider",command= self.Set_Position)
        POS_Set.grid(row=4, column=2, pady=20,padx=20)


        #Setting Angular Command and validating entry. The value is stored using Angle.get() method
        Angular_Label=tk.Label(self,text="Commande de position angulaire (en °)")
        Angular_Label.grid(row=6,column=0,pady=20)
        Angular_Entry=tk.Entry(self,textvariable=self.controller.Angle)
        Angular_Entry.grid(row=6,column=1,padx=5,pady=20)
        ANG_Set=tk.Button(self,text="Valider",command= self.Set_Angle)
        ANG_Set.grid(row=6, column=2, pady=20,padx=20)


        #Integer, used to decide wether the data will be displayed as a function of time or distance
        self.var=tk.IntVar(master)
        self.var.set(0)


        #Setting the display mode
        Display_choice_Label=tk.Label(self,text="Choix du mode d'affichage")
        Display_choice_Label.grid(row=10,column=1,pady=20)
        Display_choice_Label.config(font=("Courrier",30))
        Display_choice_time=tk.Radiobutton(self,text='Temps',variable=self.var,value=1,command=partial(self.Update_Display,controller))
        Display_choice_time.grid(row=11,column=1,pady=10)
        Display_choice_distance=tk.Radiobutton(self,text='Distance',variable=self.var,value=2,command=partial(self.Update_Display,controller))
        Display_choice_distance.grid(row=13,column=1,pady=10)



    '''Class Methods
------------------------------------------------------------------------------'''
    def Set_MSpeed(self):

        '''Validates the Movement Speed Command set by the operator
        Inputs: self
        Outputs: None
        Author : FRESLIER Clément, 10/04/2020                                 '''

        self.controller.M_Speed=self.MS_Slider.get()

        #Movement Speed Restrictions
        if (self.controller.M_Speed>0) and (self.controller.M_Speed<10):
            self.controller.M_Speed=10



    def Set_Position(self):

        '''Validates the Position Command set by the operator
        Inputs: self
        Outputs: None
        Author : FRESLIER Clément, 10/04/2020                                 '''

        self.controller.Position.get()



    '''Set Angle Command
------------------------------------------------------------------------------'''
    def Set_Angle(self):

        '''Validates the Angle Command set by the operator
        Inputs: self
        Outputs: None
        Author : FRESLIER Clément, 10/04/2020                                 '''

        self.controller.Angle.get()



    '''Updates wether data should be displayed as a function of time or distance travelled
------------------------------------------------------------------------------'''
    def Update_Display(self,controller):

        '''Updates wether data should be displayed as a function of time or distance travelled, creates the window if none is created
        Inputs: self, controller
        Outputs: None
        Author: FRESLIER Clément, 22/04/2020                                  '''

        #if no Display Page already exists
        if (self.Data_Page==None):

            controller.show_frame(Main_Page)

        if self.Data_Page:

            self.Data_Page.Update()




'''Subclass containing display
------------------------------------------------------------------------------'''
class Display_Page(tk.Toplevel):



    def __init__(self,master,controller):

        '''Class Attributes
------------------------------------------------------------------------------'''

        tk.Toplevel.__init__(self, master,bg='black')


        #controller=master class, used to call methods inherited from that class
        self.controller=controller
        self.master=master

        self.mode=controller.Get_mode()


        #String to be displayed
        self.string=""


        #Number of elements to display
        self.buffer_size=35

        #Display Label
        self.Data_Label=tk.Label(self,fg='green',bg='black')
        self.Data_Label.place(x=0,y=0)




    '''Class Methods
------------------------------------------------------------------------------'''
    def Display_Data(self):

        '''Displays Data on the Display Page
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 26/04/2020                               '''

        self.Data_Label.config(text=self.string)



    def Update(self):

        '''Takes the data stored in self.controller.data and puts them into te string attribute, accordingly to the selected mode.
           The number of data to display must be lower than the display limit, and is controlled by the local variable i.
           This method also updates the mean temperature and hygrometry calculated in Retrieve_Data and displayed in the Analysis_Page class
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 26/04/2020                                   '''

        #Number of displayed data
        i=0

        #Display mode
        self.mode=self.controller.Get_mode()

        #Data to be displayed, read from last element to first
        for Position, Angle, Date, Temperature, Hygrometry in reversed(self.controller.data):

            #if the number of displayed data is higher than limit
            if i >= self.buffer_size:
                break

            #if selected data mode is "time"
            if self.mode==1:

                self.string+= "{0} : Température= {1}°C, Hygrométrie= {2}% \n".format(Date,Temperature,Hygrometry)

            #if selected data mode is "position"
            elif self.mode==2:

                self.string+="Postion: {0}cm, {1}° :Température= {2}°C, Hygrométrie= {3}% \n".format(Position, Angle, Temperature, Hygrometry)

            i+=1

        #update mean temperature and stores it the T_str attribute of Analysis_Page class
        self.controller.frames[Analysis_Page].T_str= "Température Moyenne ={:.3f}°C".format(self.controller.avg_Temp)
        self.controller.frames[Analysis_Page].H_str= "Hygrométrie Moyenne ={:.3f}%".format(self.controller.avg_Hyg)

        #Call display methods to refresh the label's text
        self.Display_Data()
        self.controller.frames[Analysis_Page].Update_avg()

        self.string=""




'''Subclass containing the help page
------------------------------------------------------------------------------'''
class Help_Page(tk.Frame):

    Text="""                                                       Aide Interface Utilisateur Robot Veronica

        L'interface utilisateur du Robot Veronica comporte trois fenêtres principales.


        La fenêtre de droite est une fenêtre de commande, dans laquelle l'utilisateur peut entrer les consignes de position ainsi que de position
        angulaire souhaitées, relativement à la position actuelle du robot: si l'entrée vaut 0, le robot s'arrête.

        Un slider permet également de régler la vitesse du robot en cm/s. La valeur 0 correspond à une commande de vitesse nulle, et toute
        consigne de vitesse entre 1 et 10 cm/s règlera la valeur de vitesse à 10cm/s afin d'obéir au cahier des charges.

        Chaque entrée doit être validée via le bouton "valider" pour qu'elle soit prise en compte par le robot, une saisie simple ne suffit pas.

        Cette fenêtre présente enfin un bouton permettant de choisir le mode d'affichage des données reçues : en fonction du temps ou de la
        distance parcourue depuis la dernière mise en marche.



        La fenêtre de gauche est la fenêtre d'affichage des données collectées par le Robot, selon le mode d'affichage choisi.
        Les données sont de la forme suivante :
                                            Dans le format "temporel" :  YYYY/MM/DD HH:MM:SS: Temp=T°C HYG=H%
                                            Dans le format "spatial" :   Position: XX cm, YY°: Temp=T°C HYG=H%

        Si la fenêtre d'affichage est fermée, elle peut être rouverte en
                                            -rouvrant la fenêtre de commande (par Aide ou Menu Principal)
                                            -Choisissant un mode d'affichage (temps ou position)

        Le bouton "Arrêt" permet de fermer l'interface utilisateur. Le robot continuera de fonctionner jusqu'à la dernière consigne de position
        saisie puis se metta en veille (et ne transmettra plus de données) jusqu'à la prochaine activation du programme.


        L'utilisateur peut ouvrir une fenêtre d'analyse des données à l'aide du bouton correspondant sur lepanneau de commande.
        Cette nouvelle fenêtre affiche en temps réel la température et le taux d'humidité relevés par le robot en fonction de la date d'acquisition
        des données. Les températures et taux d'humidité moyen sont également affichés.

        De plus, il est possible depuis cette fenêtre de sauvegarder les données dans un fichier texte.
    """


    def __init__(self, master, controller):

        '''Class Attributes
------------------------------------------------------------------------------'''

        tk.Frame.__init__(self, master)


        #controller=master class, used to call methods inherited from that class
        self.master=master
        self.controller=controller


        #Creating Text area containing the help message
        T=tk.Text(self,height=30, width=150)
        T.grid(row=0,column=0,sticky="NSEW")
        T.insert(tk.END,self.Text)


        #Back to Home Page Button
        Back_Start_Page = tk.Button(self, text="Retour au Menu Principal",
                            command=lambda: controller.show_frame(Start_Page))
        Back_Start_Page.grid(row=1,column=0,sticky="SE")


        #Back to Interface Button
        Back_Main_Page = tk.Button(self, text="Retour à l'interface",
                            command=lambda: controller.show_frame(Main_Page))
        Back_Main_Page.grid(row=1,column=0,sticky='S')




'''Subclass containing the Analysis page, can be completed
------------------------------------------------------------------------------'''
class Analysis_Page(tk.Frame):


    def __init__(self,master,controller):

        '''Class Attributes
------------------------------------------------------------------------------'''

        tk.Frame.__init__(self, master)


        #Controller=master class, used to call methods inherited from that class
        self.master=master
        self.controller=controller


        #Page Title
        Analysis_title = tk.Label(self, text="Page d'Analyse des données",font=('Verdana',20))
        Analysis_title.place(x=600,y=0,anchor="n")


        #strings used to print mean values
        self.T_str=""
        self.H_str=""


        #Save file entry, give the name of the file data will be saved in
        self.Save_File=tk.Entry(self,width=15)
        self.Save_File.place(x=1000,y=600,anchor="e")


        #Save file button
        Save_bt = tk.Button(self, text="Sauvegarder",
                            command=self.Save_Data)
        Save_bt.place(x=900,y=600,anchor="e")


        #Average Temperature display
        self.Average_Temperature = tk.Label(self, text="Température Moyenne ={}°C".format(controller.avg_Temp),font=('Verdana',20))
        self.Average_Temperature.place(x=950,y=300,anchor="n")


        #Average Hydrometry display
        self.Average_Hygrometry = tk.Label(self, text="Hygrométrie Moyenne ={}%".format(controller.avg_Hyg),font=('Verdana',20))
        self.Average_Hygrometry.place(x=950,y=350,anchor="n")


        #Back to interface button
        Back_Interface = tk.Button(self, text="Retour à l'interface",
                            command=lambda: controller.show_frame(Main_Page))
        Back_Interface.place(x=1200,y=0,anchor="ne")


        #Help page button
        Help_bt = tk.Button(self, text="Aide",
                            command=lambda: controller.show_frame(Main_Page))
        Help_bt.place(x=1090,y=0,anchor="ne")


        #Graph to be displayed on the page
        self.plot_graph()




    '''Class Methods
------------------------------------------------------------------------------'''
    def plot_graph (self):

        '''Define the canvas in which the live graph will be plotted
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 29/04/2020                               '''

        #frame to display the graph in
        canvas_frame=tk.Frame(self,width=500,height=500)
        canvas_frame.place(x=0,y=40)

        #canvas containing the graph
        canvas = FigureCanvasTkAgg(fig, canvas_frame)
        canvas.draw()
        canvas.get_tk_widget().pack(side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        #Matplotlib toolbar
        toolbar = NavigationToolbar2Tk(canvas, self)
        toolbar.update()
        canvas._tkcanvas.pack(side=tk.TOP, fill=tk.BOTH, expand=True)



    def Update_avg(self):

        '''Update average values of temperature and hygrometry
           Inputs: self
           Ouputs: None
           Author: FRESLIER Clément, 29/04/2020                               '''

        #Setting updated average Temperature and Hydrometry in their respective labels
        self.Average_Temperature.configure(text=self.T_str)
        self.Average_Hygrometry.configure(text=self.H_str)



    def Save_Data(self):

        '''Saves data in text file
           Inputs: self
           Outputs: None
           Author: FRESLIER Clément, 29/04/2020                               '''

        #Relative path in which the file is
        rel_path = str(self.Save_File.get())

        #If a file name has been entered, set the absolute path using the SCRIPT_DIT constant
        if self.Save_File.get()!="":
            filename=os.path.join(SCRIPT_DIR, rel_path)

            #saving data in the text file
            savefile=open(filename,"a")
            for Position, Angle, Date, Temperature, Hygrometry in self.controller.data:
                savefile.write("Position= {} cm , Angle= {} ° , Date= {} , Température= {} °C, Hygrométrie = {} % \n".format(Position,Angle,Date,Temperature,Hygrometry))

            #closing the text file
            savefile.close()

            #Resetting the filename entry
            self.Save_File.delete(0,"end")
            self.Save_File.insert(0, "")
            filename=""



    def animate(self,i):

        '''refreshes graph values
        Inputs=self, i (interval, not used in the function but in the animation method)
        Outputs: None
        Author: FRESLIER Clément, 28/04/2020                                   '''

        #Variables will be stores in the corresponding lists
        Pos_List=[]
        Ang_List=[]
        Date_List=[]
        Temp_List=[]
        Hyg_List=[]

        #Storing position and formatting them to float or Datetime objects
        for Position_str, Angle_str, Date_str, Temperature_str, Hygrometry_str in self.controller.data:

            Position=float(Position_str)
            Angle=float(Angle_str)
            Temperature=float(Temperature_str)
            Hygrometry=float(Hygrometry_str)

            Date=datetime.strptime(Date_str, "%Y/%m/%d %H:%M:%S")
            Date.strftime("%H:%M:%S")

            Date_List.append(Date)
            Pos_List.append(Position)
            Ang_List.append(Angle)
            Temp_List.append(Temperature)
            Hyg_List.append(Hygrometry)


        ax1.clear()
        ax2.clear()

        delta_t=timedelta(seconds=20)

        #if there's something to plot, plot it as a function of the date with moving x axis
        if len(Date_List)!=0:
            ax1.plot_date(Date_List,Temp_List,'r-',label="Température")
            ax1.legend(loc='upper right')
            ax1.set_xlim(left=max(Date_List[0],Date_List[self.controller.x_index]-delta_t),right=(Date_List[-1]+timedelta(seconds=5)))

            #Secondary y axis
            ax2.plot_date(Date_List,Hyg_List,'g:',label="Hygrométrie")
            ax2.legend(loc="upper right",bbox_to_anchor=(1,0.95))
            self.controller.x_index+=1

        #Labels and graph Title
        ax1.set_xlabel('Temps')
        ax1.set_ylabel('Température (°C)')
        ax2.set_ylabel('Hygrométrie (%)')
        ax1.set_title('Température et Hygrométrie en fonction du temps')
        ax1.grid()





'''Function surveying if new data has arrived and, once it has, stores it in the data attribute from HMI_Veronica class
------------------------------------------------------------------------------'''
def Retrieve_Data(New_Data,HMI_Veronica):

    '''Surveys in real time if new data has arrived and if so, adds it to 'data'
       Inputs: New_Data ->list of new data, format [Position1, Angle1,Date1,Temperature1,Hyrometry1,Position2,....,True]
               HMI_Veronica ->Main class of the application
       Outputs: None
       Author: FRESLIER Clément, 26/04/2020                                   '''

    AvgT=[]
    AvgH=[]

    while True:

        #Sets the function to sleep so that it doesn't take too much resources
        sleep(0.01)

        #When new data is received
        if New_Data[-1]==True:

            New_Data.pop()

            #While the list isn't empy, stores values in relevant variables
            while New_Data:

                Hygrometry=New_Data.pop()
                Temperature=New_Data.pop()
                Date=New_Data.pop()
                Angle=New_Data.pop()
                Position=New_Data.pop()


                #stores those variables in the data list
                HMI_Veronica.data.append([Position,Angle,Date,Temperature,Hygrometry])

                AvgT.append(float(Temperature))
                AvgH.append(float(Hygrometry))

            HMI_Veronica.avg_Temp=sum(AvgT)/len(AvgT)
            HMI_Veronica.avg_Hyg=sum(AvgH)/len(AvgH)

            #if the Display Page exists, call Update method
            if  HMI_Veronica.frames[Main_Page].Data_Page:

                HMI_Veronica.frames[Main_Page].Data_Page.Update()

            #Add False to the new data list to specify that no new data is there to display, until new data is received
            New_Data.append(False)














